@@ -8,7 +8,7 @@ import typing
 from abc import ABCMeta, abstractmethod
 from collections.abc import AsyncIterable, AsyncIterator, Iterable, Iterator
 from enum import EnumMeta
-from typing import Any, overload
+from typing import Any, Generic, ParamSpec, Type, TypeVar, TypeVarTuple, overload
 
 import typing_extensions
 from _typeshed import Self
@@ -317,3 +317,45 @@ def __ne__(self, other: Any) -> bool:
 
 def __imul__(self, other: Any) -> list[str]:
     ...
+
+class UsesStringizedAnnotations:
+    def __iadd__(self, other: "UsesStringizedAnnotations") -> "typing.Self":
+        return self
+
+
+class NonGeneric1(tuple):
+    def __new__(cls: type[NonGeneric1], *args, **kwargs) -> NonGeneric1: ...
+    def __enter__(self: NonGeneric1) -> NonGeneric1: ...
+
+class NonGeneric2(tuple):
+    def __new__(cls: Type[NonGeneric2]) -> NonGeneric2: ...
+
+class Generic1[T](list):
+    def __new__(cls: type[Generic1]) -> Generic1: ...
+    def __enter__(self: Generic1) -> Generic1: ...
+
+
+### Correctness of typevar-likes are not verified.
+
+T = TypeVar('T')
+P = ParamSpec()
+Ts = TypeVarTuple('foo')
+
+class Generic2(Generic[T]):
+    def __new__(cls: type[Generic2]) -> Generic2: ...
+    def __enter__(self: Generic2) -> Generic2: ...
+
+class Generic3(tuple[*Ts]):
+    def __new__(cls: type[Generic3]) -> Generic3: ...
+    def __enter__(self: Generic3) -> Generic3: ...
+
+class Generic4(collections.abc.Callable[P, ...]):
+    def __new__(cls: type[Generic4]) -> Generic4: ...
+    def __enter__(self: Generic4) -> Generic4: ...
+
+from some_module import PotentialTypeVar
+
+class Generic5(list[PotentialTypeVar]):
+    def __new__(cls: type[Generic5]) -> Generic5: ...
+    def __enter__(self: Generic5) -> Generic5: ...
+
