@@ -1,11 +1,11 @@
 use itertools::Itertools;
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 
 use ruff_python_ast::{self as ast, CmpOp, Expr};
 use ruff_python_semantic::SemanticModel;
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -18,6 +18,22 @@ use crate::checkers::ast::Checker;
 ///
 /// If you want to check for an exact type match, use `is` or `is not`.
 ///
+/// ## Known problems
+/// When using libraries that override the `==` (`__eq__`) operator (such as NumPy,
+/// Pandas, and SQLAlchemy), this rule may produce false positives, as converting
+/// from `==` to `is` or `is not` will change the behavior of the code.
+///
+/// For example, the following operations are _not_ equivalent:
+/// ```python
+/// import numpy as np
+///
+/// np.array([True, False]) == False
+/// # array([False,  True])
+///
+/// np.array([True, False]) is False
+/// # False
+/// ```
+///
 /// ## Example
 /// ```python
 /// if type(obj) == type(1):
@@ -32,24 +48,23 @@ use crate::checkers::ast::Checker;
 /// if isinstance(obj, int):
 ///     pass
 /// ```
-#[violation]
-pub struct TypeComparison;
+#[derive(ViolationMetadata)]
+pub(crate) struct TypeComparison;
 
 impl Violation for TypeComparison {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!(
-            "Use `is` and `is not` for type comparisons, or `isinstance()` for isinstance checks"
-        )
+        "Use `is` and `is not` for type comparisons, or `isinstance()` for isinstance checks"
+            .to_string()
     }
 }
 
 /// E721
-pub(crate) fn type_comparison(checker: &mut Checker, compare: &ast::ExprCompare) {
-    for (left, right) in std::iter::once(compare.left.as_ref())
-        .chain(compare.comparators.iter())
+pub(crate) fn type_comparison(checker: &Checker, compare: &ast::ExprCompare) {
+    for (left, right) in std::iter::once(&*compare.left)
+        .chain(&compare.comparators)
         .tuple_windows()
-        .zip(compare.ops.iter())
+        .zip(&compare.ops)
         .filter(|(_, op)| matches!(op, CmpOp::Eq | CmpOp::NotEq))
         .map(|((left, right), _)| (left, right))
     {
@@ -61,9 +76,7 @@ pub(crate) fn type_comparison(checker: &mut Checker, compare: &ast::ExprCompare)
             }
 
             // Disallow the comparison.
-            checker
-                .diagnostics
-                .push(Diagnostic::new(TypeComparison, compare.range()));
+            checker.report_diagnostic(TypeComparison, compare.range());
         }
     }
 }
@@ -180,8 +193,8 @@ fn is_type(expr: &Expr, semantic: &SemanticModel) -> bool {
 /// Returns `true` if the [`Expr`] appears to be a reference to a NumPy dtype, since:
 /// > `dtype` are a bit of a strange beast, but definitely best thought of as instances, not
 /// > classes, and they are meant to be comparable not just to their own class, but also to the
-/// corresponding scalar types (e.g., `x.dtype == np.float32`) and strings (e.g.,
-/// `x.dtype == ['i1,i4']`; basically, __eq__ always tries to do `dtype(other)`).
+/// > corresponding scalar types (e.g., `x.dtype == np.float32`) and strings (e.g.,
+/// > `x.dtype == ['i1,i4']`; basically, __eq__ always tries to do `dtype(other)`).
 fn is_dtype(expr: &Expr, semantic: &SemanticModel) -> bool {
     match expr {
         // Ex) `np.dtype(obj)`
