@@ -26,6 +26,9 @@ use crate::{Applicability, Edit, Fix, FixAvailability, Violation};
 ///
 /// ## Example
 /// ```python
+/// from decimal import Decimal
+/// from fractions import Fraction
+///
 /// Decimal.from_float(4.2)
 /// Decimal.from_float(float("inf"))
 /// Fraction.from_float(4.2)
@@ -34,10 +37,13 @@ use crate::{Applicability, Edit, Fix, FixAvailability, Violation};
 ///
 /// Use instead:
 /// ```python
+/// from decimal import Decimal
+/// from fractions import Fraction
+///
 /// Decimal(4.2)
 /// Decimal("inf")
 /// Fraction(4.2)
-/// Fraction(Decimal(4.2))
+/// Fraction(Decimal("4.2"))
 /// ```
 ///
 /// ## Fix safety
@@ -287,7 +293,19 @@ fn handle_non_finite_float_special_case(
         return None;
     };
     let normalized = as_non_finite_float_string_literal(float_arg)?;
-    let replacement_text = format!(r#"{constructor_name}("{normalized}")"#);
+    let replacement_text = format!(
+        r#"{constructor_name}("{}")"#,
+        // `Decimal.from_float(float(" -nan")) == Decimal("nan")`
+        if normalized == "-nan" {
+            // Here we do not attempt to remove just the '-' character.
+            // It may have been encoded (e.g. as '\N{hyphen-minus}')
+            // in the original source slice, and the added complexity
+            // does not make sense for this edge case.
+            "nan"
+        } else {
+            normalized
+        }
+    );
     Some(Edit::range_replacement(replacement_text, call.range()))
 }
 
