@@ -8,33 +8,41 @@ use crate::SemanticModel;
 /// import sys
 ///
 /// sys.path.append("../")
+/// sys.path += ["../"]
 /// ```
 pub fn is_sys_path_modification(stmt: &Stmt, semantic: &SemanticModel) -> bool {
-    let Stmt::Expr(ast::StmtExpr { value, range: _ }) = stmt else {
-        return false;
-    };
-    let Expr::Call(ast::ExprCall { func, .. }) = value.as_ref() else {
-        return false;
-    };
-    semantic
-        .resolve_qualified_name(func.as_ref())
-        .is_some_and(|qualified_name| {
-            matches!(
-                qualified_name.segments(),
-                [
-                    "sys",
-                    "path",
-                    "append"
-                        | "insert"
-                        | "extend"
-                        | "remove"
-                        | "pop"
-                        | "clear"
-                        | "reverse"
-                        | "sort"
-                ]
-            )
-        })
+    match stmt {
+        Stmt::Expr(ast::StmtExpr {
+            value,
+            range: _,
+            node_index: _,
+        }) => match value.as_ref() {
+            Expr::Call(ast::ExprCall { func, .. }) => semantic
+                .resolve_qualified_name(func.as_ref())
+                .is_some_and(|qualified_name| {
+                    matches!(
+                        qualified_name.segments(),
+                        [
+                            "sys",
+                            "path",
+                            "append"
+                                | "insert"
+                                | "extend"
+                                | "remove"
+                                | "pop"
+                                | "clear"
+                                | "reverse"
+                                | "sort"
+                        ]
+                    )
+                }),
+            _ => false,
+        },
+        Stmt::AugAssign(ast::StmtAugAssign { target, .. }) => semantic
+            .resolve_qualified_name(map_subscript(target))
+            .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["sys", "path"])),
+        _ => false,
+    }
 }
 
 /// Returns `true` if a [`Stmt`] is an `os.environ` modification, as in:
@@ -92,7 +100,12 @@ pub fn is_os_environ_modification(stmt: &Stmt, semantic: &SemanticModel) -> bool
 /// matplotlib.use("Agg")
 /// ```
 pub fn is_matplotlib_activation(stmt: &Stmt, semantic: &SemanticModel) -> bool {
-    let Stmt::Expr(ast::StmtExpr { value, range: _ }) = stmt else {
+    let Stmt::Expr(ast::StmtExpr {
+        value,
+        range: _,
+        node_index: _,
+    }) = stmt
+    else {
         return false;
     };
     let Expr::Call(ast::ExprCall { func, .. }) = value.as_ref() else {
@@ -102,3 +115,44 @@ pub fn is_matplotlib_activation(stmt: &Stmt, semantic: &SemanticModel) -> bool {
         .resolve_qualified_name(func.as_ref())
         .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["matplotlib", "use"]))
 }
+
+/// Returns `true` if a [`Stmt`] is a `pytest.importorskip()` call, as in:
+/// ```python
+/// import pytest
+///
+/// pytest.importorskip("foo.bar")
+/// ```
+pub fn is_pytest_importorskip(stmt: &Stmt, semantic: &SemanticModel) -> bool {
+    let Stmt::Expr(ast::StmtExpr { value, .. }) = stmt else {
+        return false;
+    };
+    let Expr::Call(ast::ExprCall { func, .. }) = value.as_ref() else {
+        return false;
+    };
+
+    semantic
+        .resolve_qualified_name(func.as_ref())
+        .is_some_and(|qualified_name| {
+            matches!(qualified_name.segments(), ["pytest", "importorskip"])
+        })
+}
+
+/// Returns `true` if a [`Stmt`] is a dynamic modification of the Python
+/// module search path, e.g.,
+/// ```python
+/// import site
+///
+/// site.addsitedir(...)
+/// ```
+pub fn is_site_sys_path_modification(stmt: &Stmt, semantic: &SemanticModel) -> bool {
+    if let Stmt::Expr(ast::StmtExpr { value, .. }) = stmt {
+        if let Expr::Call(ast::ExprCall { func, .. }) = value.as_ref() {
+            return semantic
+                .resolve_qualified_name(func.as_ref())
+                .is_some_and(|qualified_name| {
+                    matches!(qualified_name.segments(), ["site", "addsitedir"])
+                });
+        }
+    }
+    false
+}
